#!/bin/bash

# Copyright (c) xFusion Digital Technologies Co., Ltd. 2022. All rights reserved.
# Description: ip conflict check
# Author: xFusion Digital Technologies
# Create: 2022-1-17

G_IPS=""
G_TMP_DIR="/tmp/ipcc_${RANDOM}_$$"
G_TMP_RESULT="$G_TMP_DIR/result"
trap '[ -n "${G_TMP_DIR}" ] && rm -rf "${G_TMP_DIR}"' EXIT

# **************************************************************************** #
# Function Name: fn_echo_param_error
# Description: echo param error to kmsg
# Parameter: 
# Return: 0-succ 1-fail
# **************************************************************************** #
fn_echo_param_error()
{
    echo "Param Error: please call the script like this $0 -i ethx -d 192.168.0.1" > /dev/kmsg
}

# **************************************************************************** #
# Function Name: fn_deal_with_one_interface
# Description: deal with all ip of one interface
# Parameter: $1: interface name
# Return: 0-succ 1-fail
# **************************************************************************** #
fn_deal_with_one_interface()
{
    local INTERFACE=$1

    for IP in $(/sbin/ip add show "${INTERFACE}"| grep -E "[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}" | awk '{print$2}' | awk -F \/ '{print $1}')
    do
        fn_deal_with_one_ip ${INTERFACE} ${IP}
    done
    
}

# **************************************************************************** #
# Function Name: fn_deal_with_one_ip
# Description: deal with one ip of one interface
# Parameter: $1: interface name $2:ip address
# Return: 0-succ 1-fail
# **************************************************************************** #
fn_deal_with_one_ip()
{
    local INTERFACE=$1
    local IP=$2
    
    fn_test_ip_duplicate ${INTERFACE} ${IP}
    ret=$?
    if [ 1 -eq "$ret" ]; then
        echo "Warning: ip ${IP} on ${INTERFACE} is duplicate." > /dev/kmsg
    elif [ 2 -eq "$ret" ]; then
        echo "The interface ${INTERFACE} might be down or unknown while ip conflict checking." > /dev/kmsg
    fi
    return $ret
}

# **************************************************************************** #
# Function Name: fn_test_ip_duplicate
# Description: test whether the ip on the interface is duplicate
# Parameter: $1: interface name $2:ip address
# Return: 0-succ 1-fail
# **************************************************************************** #
fn_test_ip_duplicate()
{
    # use interface without subinterface
    local INTERFACE=$1
    local IP=$2
    local count=0
    local interface_mac=""

    interface_mac=`/sbin/ip add show ${INTERFACE}|grep "link/ether"|awk '{print $2}'`
    for ip in $(echo "${G_IPS}")
    do
        if [ "${ip}" == "${IP}" ]; then
            count=$((count+1))
        fi
    done
    
    if [ "${count}" -gt 0 ]; then
        for if in `/sbin/ip add show|grep -E "^[0-9]*:"|cut -d ":" -f2`
        do
            if=`echo $if|awk -F "@" '{print $1}'`
            ip addr show ${if} | grep -w 'inet' | awk '{print $2}' |grep -wqF "${IP}"
            if [ $? -eq 0 -a ${if} != ${INTERFACE} ];then
                mac_address=`/sbin/ip add show ${if}|grep "link/ether"|awk '{print $2}'`
                if [ "${interface_mac}" == "${mac_address}" ];then
                    continue
                fi
                echo "Target link-layer address: ${mac_address}"
                echo "Target link-layer address: ${mac_address}" > /dev/kmsg
                return 1
            fi
        done
    fi
    
    mkdir -m 700 "${G_TMP_DIR}"
    if [ $? -ne 0 ];then
        echo "[$0]mkdir fail" >/dev/kmsg
        return 0
    fi
    arping -I ${INTERFACE} -D -c 3 ${IP} > ${G_TMP_RESULT} 2>&1
    ret=$?

    mac_address=`cat ${G_TMP_RESULT}|grep "Unicast reply from"|awk -F '[\\\[\\\]]' '{print $2}'`
    rm -rf ${G_TMP_DIR}
    if [ "${mac_address}x" != "x" ];then
        echo "Target link-layer address: ${mac_address}"
        echo "Target link-layer address: ${mac_address}" > /dev/kmsg
    fi

    return $ret
}

# **************************************************************************** #
# Function Name: fn_get_ips_of_other_ifs
# Description: get all ips except the interface from params
# Parameter: $1: interface name
# Return: 0-succ 1-fail
# **************************************************************************** #
fn_get_all_ips()
{
    G_IPS=$(/sbin/ip add show | grep -E "[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}" | awk '{print$2}' | awk -F \/ '{print $1}')
}

# **************************************************************************** #
# Function Name: fn_broadcast_garp
# Description: broadcast GARPs for one IP is set.
# Parameter: $1: interface name $2:ip address
# Return: N/A
# **************************************************************************** #
fn_broadcast_garp()
{
    # use interface without subinterface
    local INTERFACE=$1
    local IP=$2
    local count=0

    if [ "${IP}" == "0.0.0.0" ]; then
        for IP in $(/sbin/ip add show "${INTERFACE}"| grep -E "[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}" | awk '{print$2}' | awk -F \/ '{print $1}')
        do
            arping -I ${INTERFACE} -s ${IP} -c 1 ${IP} >/dev/null 2>&1 &
        done
    else
        arping -I ${INTERFACE} -s ${IP} -c 1 ${IP} >/dev/null 2>&1 &
    fi

    return 0
}

# ------------------------------------main-------------------------------------#

# 参数解析，由于可能通过C代码来调用该脚本，参数可能作为一个字符串传进来
# 需要手工解析
if [ $# -eq 4 ]; then
    PARAM1=$1
    PARAM2=$2
    PARAM3=$3
    PARAM4=$4
elif [ $# -eq 1 ]; then
    PARAMS=$@
    PARAM1=$(echo "${PARAMS}" | awk '{print $1}')
    PARAM2=$(echo "${PARAMS}" | awk '{print $2}')
    PARAM3=$(echo "${PARAMS}" | awk '{print $3}')
    PARAM4=$(echo "${PARAMS}" | awk '{print $4}')
else
    fn_echo_param_error
    exit 1
fi

if [ "${PARAM1}" != "-i" -o "${PARAM3}" != "-d" ]; then
    fn_echo_param_error
    exit 1
fi

# 判断是否是合法的IP地址
TEST_PARAM4=$(echo ${PARAM4} | grep -E "^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}$")
if [ -z "${TEST_PARAM4}" ]; then
    fn_echo_param_error
    exit 1
fi

# 对接口名进行处理，只保留真正的接口名，忽略子接口名
PARAM2=$(echo "${PARAM2}" | awk -F \: '{print $1}')

# 获取本机所有IP
fn_get_all_ips

# 广播一次免费ARP报文
fn_broadcast_garp "${PARAM2}" "${PARAM4}"

# 若传入IP为0.0.0.0，对该网口所有IP进行检测
if [ "${PARAM4}" == "0.0.0.0" ]; then
    fn_deal_with_one_interface "${PARAM2}"
else
    fn_deal_with_one_ip "${PARAM2}" "${PARAM4}"
    if [ $? -eq 1 ]; then
        # 指定IP进行的冲突检测，如果存在冲突，固定返回值为2，其他情况请不要占用此返回值
        exit 2
    fi
fi
    
exit 0

