#!/bin/bash

# Copyright (c) xFusion Digital Technologies Co., Ltd. 2022. All rights reserved.
# Description: ipv6 conflict check
# Author: xFusion Digital Technologies
# Create: 2022-1-17

G_IPS=""
G_TMP_DIR="/tmp/ipv6cc_${RANDOM}_$$"
G_TMP_RESULT="$G_TMP_DIR/result"
trap '[ -n "${G_TMP_DIR}" ] && rm -rf "${G_TMP_DIR}"' EXIT

# **************************************************************************** #
# Function Name: fn_echo_param_error
# Description: echo param error to kmsg
# Parameter: 
# Return: 0-succ 1-fail
# **************************************************************************** #
fn_echo_param_error()
{
    echo "Param Error: please call the script like this $0 -i ethx -d xxxx::xxxx:xxxx:xxxx:xxxx" > /dev/kmsg
}

# **************************************************************************** #
# Function Name: fn_deal_with_one_ip
# Description: deal with one ip of one interface
# Parameter: $1: interface name $2:ip address
# Return: 0-succ 1-fail
# **************************************************************************** #
fn_deal_with_one_ip()
{
    local INTERFACE=$1
    local IP=$2

    python3 -c  "import socket;print (socket.inet_ntop(socket.AF_INET6, socket.inet_pton(socket.AF_INET6, '$IP')));" &>/dev/null
    if [ $? -ne 0 ];then
	echo "[$0] ipv6 address ${IP} is illegal" >/dev/kmsg
	exit 1
    fi

    IP=$(python3 -c  "import socket;print (socket.inet_ntop(socket.AF_INET6, socket.inet_pton(socket.AF_INET6, '$IP')));")
    
    fn_test_ip_duplicate ${INTERFACE} ${IP}
    ret=$?
    if [ 1 -eq "$ret" ]; then
        echo "Warning: ip ${IP} on ${INTERFACE} is duplicate." > /dev/kmsg
    elif [ 2 -eq "$ret" ]; then
        echo "The interface ${INTERFACE} or ${IP} is not correct." > /dev/kmsg
        exit 1
    fi
    return $ret
}

# **************************************************************************** #
# Function Name: fn_test_ip_duplicate
# Description: test whether the ip on the interface is duplicate
# Parameter: $1: interface name $2:ip address
# Return: 0-succ 1-fail
# **************************************************************************** #
fn_test_ip_duplicate()
{
    # use interface without subinterface
    local INTERFACE=$1
    local IP=$2
    local count=0
    local interface_mac=""

    interface_mac=`/sbin/ip add show ${INTERFACE}|grep "link/ether"|awk '{print $2}'`
    for ip in $(echo "${G_IPS}")
    do
        if [ "${ip}" == "${IP}" ]; then
            count=$((count+1))
        fi
    done
    
    if [ "${count}" -gt 0 ]; then
        for if in `/sbin/ip add show|grep -E "^[0-9]*:"|cut -d ":" -f2`
        do
            if=`echo $if|awk -F "@" '{print $1}'`
            ip add show ${if}|grep -q " ${IP}[ |/]"
            if [ $? -eq 0 -a ${if} != ${INTERFACE} ];then
                mac_address=`ip add show ${if}|grep "link/ether"|awk '{print $2}'`
                if [ "${interface_mac}" == "${mac_address}" ];then
                        continue
                fi
                if [[ ${IP} =~ ^fe80::.* ]];then
                        continue
                fi
                echo "Target link-layer address: ${mac_address}"
                return 1
            fi
        done
    fi
    
    mkdir -m 700 "${G_TMP_DIR}"
    if [ $? -ne 0 ];then
        echo "[$0]mkdir fail" >/dev/kmsg
        return 0
    fi
    sleep 1
    ndisc6 ${IP} ${INTERFACE} > ${G_TMP_RESULT}
    ret=$?

    cat ${G_TMP_RESULT}|grep "Target link-layer address"
    rm -rf ${G_TMP_DIR}

    if [ $ret == 0 ];then
            ret=1
    elif [ $ret == 2 ];then
            ret=0
    elif [ $ret == 1 ];then
            ret=2
    fi
    
    return $ret
}

# **************************************************************************** #
# Function Name: fn_get_ips_of_other_ifs
# Description: get all ips except the interface from params
# Parameter: $1: interface name
# Return: 0-succ 1-fail
# **************************************************************************** #
fn_get_all_ips()
{
    G_IPS=$(/sbin/ip add show | grep -E "inet6" | awk '{print$2}' | awk -F \/ '{print $1}')
}

# ------------------------------------main-------------------------------------#

# 参数解析，由于可能通过C代码来调用该脚本，参数可能作为一个字符串传进来
# 需要手工解析
if [ $# -eq 4 ]; then
    PARAM1=$1
    PARAM2=$2
    PARAM3=$3
    PARAM4=$4
elif [ $# -eq 1 ]; then
    PARAMS=$@
    PARAM1=$(echo "${PARAMS}" | awk '{print $1}')
    PARAM2=$(echo "${PARAMS}" | awk '{print $2}')
    PARAM3=$(echo "${PARAMS}" | awk '{print $3}')
    PARAM4=$(echo "${PARAMS}" | awk '{print $4}')
else
    fn_echo_param_error
    exit 1
fi

if [ "${PARAM1}" != "-i" -o "${PARAM3}" != "-d" ]; then
    fn_echo_param_error
    exit 1
fi

# 对接口名进行处理，只保留真正的接口名，忽略子接口名
PARAM2=$(echo "${PARAM2}" | awk -F \: '{print $1}')

ifconfig ${PARAM2} 2>/dev/null | grep -wq 'RUNNING'
if [ $? -ne 0 ];then
    echo "The interface ${PARAM2} might be down or unknown while ip conflict checking." > /dev/kmsg
    exit 1
fi

# 获取本机所有IP
fn_get_all_ips

fn_deal_with_one_ip "${PARAM2}" "${PARAM4}"
if [ $? -eq 1 ]; then
        # 指定IP进行的冲突检测，如果存在冲突，固定返回值为2，其他情况请不要占用此返回值
        exit 2
fi

exit 0

